﻿using System;
using System.IO;
using System.Threading;
using Microsoft.SPOT;
using Microsoft.SPOT.Hardware;
using GHIElectronics.NETMF.FEZ;
using GHIElectronics.NETMF.Hardware;
using GHIElectronics.NETMF.System;
using GHIElectronics.NETMF.IO;

namespace Crystal_Ball
{
    public class Program
    {
        static PWM Siluett;
        static PWM Cathode;
        static Int64 LastTrigger = DateTime.Now.Ticks;
        static byte[] buffer = new byte[30 * 1024];
        static AnalogOut aout = new AnalogOut(AnalogOut.Pin.Aout0);
        static FEZ_Components.ServoMotor Servo;

        public static void Main()
        {
            Debug.EnableGCMessages(false);
            Servo = new FEZ_Components.ServoMotor(FEZ_Pin.Digital.Di4);
            Cathode = new PWM((PWM.Pin)FEZ_Pin.PWM.Di5);
            Siluett = new PWM((PWM.Pin)FEZ_Pin.PWM.Di6);
            Cathode.Set(10000, 100);
            Siluett.Set(10000, 0);
            Servo.SetPosition(90);

            PersistentStorage sd = new PersistentStorage("SD");
            sd.MountFileSystem();

            PWM[] RGB = new PWM[3];
            RGB[0] = new PWM((PWM.Pin)FEZ_Pin.PWM.Di10);
            RGB[1] = new PWM((PWM.Pin)FEZ_Pin.PWM.Di9);
            RGB[2] = new PWM((PWM.Pin)FEZ_Pin.PWM.Di8);

            int[] old = new int[] { 0, 0, 0 };
            int I, Color, Value;

            Random NumberGenerator = new Random();

            InterruptPort PIR = new InterruptPort((Cpu.Pin)FEZ_Pin.Interrupt.Di7, true, Port.ResistorMode.PullUp, Port.InterruptMode.InterruptEdgeLow);
            PIR.OnInterrupt += new NativeEventHandler(PIR_OnInterrupt);

            while (true)
            {
                Color = NumberGenerator.Next(3);
                Value = NumberGenerator.Next(100);

                if (old[Color] < Value)
                {
                    for (I = old[Color]; I < Value; I++)
                    {
                        RGB[Color].Set(10000, (byte)I);
                        Thread.Sleep(10);
                    }
                }
                else
                {
                    for (I = old[Color]; I > Value; I--)
                    {
                        RGB[Color].Set(10000, (byte)I);
                        Thread.Sleep(10);
                    }
                }
                old[Color] = Value;

                Thread.Sleep(100);
            }
        }

        static void PIR_OnInterrupt(uint port, uint state, DateTime time)
        {
            Int64 TimeStamp = (DateTime.Now.Ticks - LastTrigger) / 10000;

            if (state == 0)
            {
                if (TimeStamp > 5000) // Ignore multible trip for 5 seconds
                {
                    // Switch light
                    Cathode.Set(10000, 0);
                    Siluett.Set(10000, 100);

                    // Play scream
                    Thread Scream;
                    Scream = new Thread(PlayFile);
                    Scream.Start();

                    // Animate eye movement
                    Thread.Sleep(300);
                    Servo.SetPosition(130);
                    Thread.Sleep(900);
                    Servo.SetPosition(50);
                    Thread.Sleep(900);
                    Servo.SetPosition(90);
                    Thread.Sleep(500);

                    // Restore light
                    for (byte I = 100; I > 0; I--)
                    {
                        Siluett.Set(10000, I);
                        Thread.Sleep(10);
                    }
                    Siluett.Set(10000, 0);

                    for (byte I = 0; I < 100; I++)
                    {
                        Cathode.Set(10000, I);
                        Thread.Sleep(10);
                    }
                    Cathode.Set(10000, 100);

                    // Reset servo position
                    //Servo.SetPosition(90);

                    LastTrigger = DateTime.Now.Ticks;
                }
            }
        }

        public static void PlayFile()
        {
            FileStream file = new FileStream("\\SD\\Skrik.wav", FileMode.Open, FileAccess.Read);
            int length = file.Read(buffer, 0, buffer.Length);
            WAVE wav = new WAVE(buffer);
            aout.Set(buffer, wav.GetDataIndex(), wav.GetDataSize(), wav.GetSampleRate());
            file.Close();
        }

    }
}
